#!/usr/bin/perl
#$Id: createtable.pl,v 3.4 2013-10-30 19:28:30 boris Exp $
#
#
#
#    
#

use Getopt::Std; 

$usage = 
"Usage: createtable.pl [-d] [-s|-m] [-t answers_file]  [-o results] \n";

require 'parameters.pl';
require 'subroutines.pl';

die $usage unless getopts('t:o:dsm');

$DEBUG=$opt_d;
$SHORT=$opt_s;
$MEDIUM=$opt_m;

#        .
$MINQUEST=1e99;
$MAXQUEST=-1;
for (my $r=1; $r<=$ROUNDS; $r++) {
    if ($MINQUEST>$MINQUEST[$r]) {
        $MINQUEST=$MINQUEST[$r];
    }
    if ($MAXQUEST<$MAXQUEST[$r]) {
        $MAXQUEST=$MAXQUEST[$r];
    }
}
if ($MINQUEST < 0 || $MAXQUEST < 0) {
   die "    !\n";
}

my %teams;
my @answers;
my @ratings;
#
#   
#
if ($opt_t) {
    die "Cannot open $opt_t\n" unless open(INFILE,$opt_t);
} else {
   *INFILE=*STDIN;
}
    
readhash(\@answers);
close(INFILE);


#
#    
#
if ($opt_o) { 
    die "Cannot open $opt_o\n" unless open(OUTFILE,">$opt_o");
    select OUTFILE;
}

#
#      
#
my @nopoint_questions; 			#   ޣ 
my @unknowns; 				#    '?'
for (my $r=1; $r<=$ROUNDS; $r++) {
    if ($DEBUG) {
	print STDERR " $r\n";
    }

    #
    #     (Ivan A Derzhanski <iad58@mail.ru>)
    #
    for ($i=$MINQUEST;$i<=$MAXQUEST;$i++) {
	foreach $answer (keys %{$answers[$i]}) {
	    %{$answers[$i]->{$answer}->{teams}} = ();
	}
    }

    #
    #      
    #
    foreach my $file ( @{$FILES[$r]} ) {
	open (INFILE, $file);
	readmail(\%teams);
	collect_answers(\%teams,\@answers);
	close (INFILE);
    }

    #
    #  (  !)  ,   ޣ
    #   .       ,    
    #      .
    #
    foreach my $num ( @{$NOCOUNT[$r]} )
    {
	#      ,     
	# ޣ,  - .   ,   
	# "> 0",      "-0".
    	if ($num =~ /^\d+/) {
	    push @nopoint_questions, $num; 
	} else {
            @nopoint_questions = grep( $_ != -$num, @nopoint_questions );
	}
    }
    #    ,  ӣ  .
    my %seen;
    @nopoint_questions = grep( !$seen{$_}++, @nopoint_questions );

    #
    #   
    #
    rate_questions(\%teams,\@answers,\@ratings,$r);

    #
    #      
    #
    find_scores(\%teams,\@answers,\@ratings,$r,0.01,\@nopoint_questions);

    #
    #      ...
    #
    #
    #  
    #
    #        ("++-++ +-+-+ -+--+")
    my $COL=5;

    print "  $NAME[$r]\n";
    if ( @nopoint_questions ) {
	#  ޣ  -   .
	print  " ޣ " . (scalar(@nopoint_questions)>1 ? "" : "") 
		. ": " . join(", ", sort {$a<=>$b} @nopoint_questions) . "\n";
    }

    printf "%5s ", "N";
    for ($i=$MINQUEST[$r];$i<=$MAXQUEST[$r];$i++)
    {
	if ($SHORT) {
	    printf("%1d",$i%10);
	} elsif ($MEDIUM) {
            if ($i == int($i/$COL)*$COL || $i==$MAXQUEST[$r]) {
		printf "%-3d", $i;
            } else {
		printf "%s", ".";
            }
	} else {
	    printf "%3d",$i;
	}
    }
    printf "%6s","";
    printf "%4s","";
    printf " ";
    print "\n";

    #
    #   
    #
    foreach $team (sort 
		   {
		       $teams{$b}->{score} <=> $teams{$a}->{score}
		       or
			   $teams{$b}->{rating} <=> $teams{$a}->{rating}
		   } keys %teams
		   )
    {
	printf "%5d ",$teams{$team}->{regnum};
	for ($i=$MINQUEST[$r];$i<=$MAXQUEST[$r];$i++)
	{
	    my $answer = $teams{$team}->{answers}[$i];
	    my $score = $answers[$i]->{$answer}->{score};
	    $score = '-' unless $score;

	    #    ޣ .
	    if ( $score eq '+' && grep($_==$i, @nopoint_questions) ) {
		$score = 'X';
	    }

	    #  ,         ;-).
	    if ( $score eq '?' ) {
		my $unk = {
			    question => $i,
			    score => $score,
			    team  => $team,
			    regnum => $teams{$team}->{regnum},
			    round => $r,
			  };
		push( @unknowns, $unk );
	    }

	    #  -!  .
	    if ($SHORT) {
		printf "%1s",$score;
	    } elsif ($MEDIUM) {
		printf "%1s",$score;
		print "  " if ($i == int($i/$COL)*$COL || $i==$MAXQUEST[$r]);
	    } else {
		printf "%3s", $score;
	    }
	}
	printf "%6.2f",$teams{$team}->{score};
	printf "%4s",$teams{$team}->{rating};
	print " $team";
	print "\n";
    }

    #
    #    -    .
    #
    if ($SHORT || $MEDIUM) {
        my $nline = 24; 			#  $nline   
	print "\n\n";
	for ($j=0; $j<$MAXQUEST[$r]/$nline; $j++) {
	    my $max = $j*$nline + $nline;
	    if ($max>$MAXQUEST[$r]) {
		$max=$MAXQUEST[$r];
	    }
	    for ($i=$MINQUEST[$r]+$j*$nline; $i<=$max; $i++){
		printf("%3s",$i);
	    }
	    printf "\n";
	    for ($i=$MINQUEST[$r]+$j*$nline; $i<=$max; $i++){
		printf("%3s",$ratings[$i]);
	    }
	    printf "\n";
	}
    } else {
	#   -   .
	printf "%5s ", "";
	for ($i=$MINQUEST[$r]; $i<=$MAXQUEST[$r]; $i++)
	{
	    printf "%3s", $ratings[$i];
	}
	print "\n";
    }
    print "\n";
    print "\n"; 		# LG: One more for better readability
}


#      .
if ( @unknowns ) {
    print STDERR "!      '+'  '-':\n";
    foreach my $unk ( @unknowns ) {
	print STDERR "\t $unk->{team} ($unk->{regnum}), "
		. " $unk->{question} ($NAME[$unk->{round}]): "
		. "$unk->{score}\n";
    }
}
#          .
check_dup_numbers(\%teams);
